﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const generator_1 = require("@kitajs/generator");
const parser_1 = require("@kitajs/parser");
const core_1 = require("@oclif/core");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const base_1 = require("../util/base");
const sponsor_1 = require("../util/sponsor");
class Build extends base_1.BaseKitaCommand {
    static description = 'Analyses your backend searching for routes and bakes it into the runtime.';
    static examples = [
        {
            command: '<%= config.bin %> <%= command.id %> -c kita.config.js',
            description: 'Builds your backend with a custom config file.'
        },
        {
            command: '<%= config.bin %> <%= command.id %> -d',
            description: 'Fast checks your backend for errors without generating the runtime.'
        }
    ];
    static flags = {
        'dry-run': core_1.Flags.boolean({
            char: 'd',
            description: 'Skips generation process and only type-checks your files.',
            default: false,
            helpGroup: 'build'
        }),
        dts: core_1.Flags.boolean({
            char: 'D',
            description: 'Skips emitting declaration files (d.ts).',
            default: true,
            allowNo: true,
            helpGroup: 'build',
            exclusive: ['dry-run']
        }),
        reset: core_1.Flags.boolean({
            char: 'r',
            description: 'Removes previous generated files before each build.',
            default: false,
            helpGroup: 'build',
            exclusive: ['dry-run']
        })
    };
    async run() {
        (0, sponsor_1.printSponsor)(this);
        const { flags } = await this.parse(Build);
        const { config, compilerOptions } = this.parseConfig(flags, {
            declaration: flags.dts
        });
        await this.prepareFirstRun(config, compilerOptions);
        core_1.ux.action.start('Warming up', '', {
            stdout: true,
            style: 'clock'
        });
        const formatter = flags['dry-run'] ? undefined : new generator_1.KitaFormatter(config);
        const parser = parser_1.KitaParser.create(config, compilerOptions, 
        // Prefer already looked up files instead of walking the folder again
        compilerOptions.rootNames, 
        // Dry runs should not generate any files
        formatter);
        core_1.ux.action.stop((0, chalk_1.default) `{cyan Ready to build!}`);
        const diagnostics = await this.runParser(parser, formatter, flags.reset, config);
        if (diagnostics.length > 0) {
            this.error((0, chalk_1.default) `{red Finished with errors!}`);
        }
        if (parser.getRouteCount() === 0) {
            this.warn((0, chalk_1.default) `{yellow No routes were found!}`);
        }
        if (process.stdout.isTTY) {
            if (flags['dry-run']) {
                this.log((0, chalk_1.default) `{green \nNo errors were found!}`);
            }
            else {
                this.log((0, chalk_1.default) `{green \nRuntime is ready to use!}`);
            }
        }
    }
}
exports.default = Build;
//# sourceMappingURL=build.js.map