﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const sponsor_1 = require("../util/sponsor");
const AvailableTemplates = [
    { name: 'Kita', value: 'kita' },
    { name: 'Kita + JSX + Tailwind', value: 'kita-jsx' }
];
const TemplatesDir = node_path_1.default.resolve(__dirname, '../../templates');
class Create extends core_1.Command {
    static description = 'Scaffolds a new project with Kita';
    static examples = [
        {
            command: '<%= config.bin %> <%= command.id %> -n mybackend',
            description: 'Scaffolds a project called mybackend.'
        }
    ];
    static flags = {
        name: core_1.Flags.string({
            char: 'n',
            description: 'The name of the project.'
        }),
        dir: core_1.Flags.string({
            char: 'd',
            description: 'The directory to create the project in.'
        }),
        template: core_1.Flags.string({
            char: 't',
            description: 'The template to use.',
            options: AvailableTemplates.map((t) => t.value)
        }),
        yes: core_1.Flags.boolean({
            char: 'y',
            description: 'Skips the prompts and uses the defaults.',
            default: false
        })
    };
    async run() {
        (0, sponsor_1.printSponsor)(this);
        const { flags } = await this.parse(Create);
        if (!process.stdout.isTTY) {
            this.error('This command requires an interactive terminal.', { exit: 1 });
        }
        const answers = await inquirer_1.default.prompt([
            {
                type: 'input',
                name: 'projectName',
                message: 'What is the name of your project?',
                default: flags.name || node_path_1.default.basename(process.cwd()),
                when: !flags.yes,
                askAnswered: true
            },
            {
                type: 'input',
                name: 'directory',
                message: 'Where would you like to create your project?',
                default: flags.dir || './' /* More readable */,
                when: !flags.yes,
                askAnswered: true,
                filter: (input) => node_path_1.default.resolve(input)
            },
            {
                type: 'list',
                name: 'template',
                message: 'Which template would you like to use?',
                default: flags.template || 'kita',
                choices: [...AvailableTemplates, { name: 'Others', disabled: 'Coming soon', short: 'Others' }],
                when: !flags.yes,
                askAnswered: true
            }
        ], {
            projectName: node_path_1.default.basename(process.cwd()),
            directory: node_path_1.default.resolve(flags.dir || ''),
            template: 'kita'
        });
        // Recursively creates the directory
        await node_fs_1.default.promises.mkdir(answers.directory, { recursive: true });
        // Ensures there's no conflict with existing files
        const files = await node_fs_1.default.promises.readdir(answers.directory);
        if (files.length > 0) {
            const { overwrite } = await inquirer_1.default.prompt([
                {
                    type: 'confirm',
                    name: 'overwrite',
                    message: 'The directory is not empty. Do you want to overwrite it?',
                    default: false
                }
            ]);
            if (!overwrite) {
                this.error('Directory is not empty. Exiting.', { exit: 1 });
            }
        }
        core_1.ux.action.start('Creating project', 'This may take a few seconds', {
            stdout: true,
            style: 'clock'
        });
        // copies the template to the selected directory
        await node_fs_1.default.promises.cp(node_path_1.default.resolve(TemplatesDir, answers.template), answers.directory, { recursive: true });
        // replaces package.json and reame @kitajs/template with the project name
        const packageJsonPath = node_path_1.default.resolve(answers.directory, 'package.json');
        const packageJson = JSON.parse(await node_fs_1.default.promises.readFile(packageJsonPath, 'utf-8'));
        packageJson.name = answers.projectName === '.' ? node_path_1.default.basename(process.cwd()) : answers.projectName;
        await node_fs_1.default.promises.writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
        const readmePath = node_path_1.default.resolve(answers.directory, 'README.md');
        const readme = await node_fs_1.default.promises.readFile(readmePath, 'utf-8');
        await node_fs_1.default.promises.writeFile(readmePath, readme.replace('@kitajs/template', answers.projectName));
        core_1.ux.action.stop((0, chalk_1.default) `{cyan Project created successfully!}`);
        const relativeness = node_path_1.default.relative(process.cwd(), answers.directory);
        this.log((0, chalk_1.default) `
To get started, run the following commands:

{gray # Globally install kita cli if you haven't already.}
{cyan $} npm i -g @kitajs/cli
${relativeness ? (0, chalk_1.default) `\n{cyan $} cd {yellow ${relativeness}}` : ''}
{cyan $} npm install
{cyan $} npm run build
{cyan $} npm run dev

{cyan Happy hacking!}

To learn more about {${chalk_1.default.level > 1 ? `hex('#b58d88')` : 'yellow'} Kita}, visit our website:
https://kita.js.org`);
    }
}
exports.default = Create;
//# sourceMappingURL=create.js.map