﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseKitaCommand = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@kitajs/common");
const generator_1 = require("@kitajs/generator");
const parser_1 = require("@kitajs/parser");
const core_1 = require("@oclif/core");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const config_1 = require("./config");
const diagnostics_1 = require("./diagnostics");
class BaseKitaCommand extends core_1.Command {
    static baseFlags = {
        config: core_1.Flags.file({
            char: 'c',
            exists: true,
            description: 'Path to your kita.config.js file, if any.',
            helpGroup: 'global'
        }),
        cwd: core_1.Flags.directory({
            description: 'Sets the current working directory for your command.',
            required: false,
            helpGroup: 'global',
            exists: true
        })
    };
    parseConfig(flags, extension, useUx = true) {
        const config = (0, config_1.readConfig)(String(flags.cwd ?? process.cwd()), this.error, String(flags.config ?? ''), extension, useUx);
        return {
            config,
            compilerOptions: (0, common_1.readCompilerOptions)(config.tsconfig)
        };
    }
    async resetRuntime(config, copyRuntime = true) {
        core_1.ux.action.start('Clearing runtime', '', {
            stdout: true,
            style: 'clock'
        });
        await fs_1.default.promises.rm(config.runtimePath, { recursive: true });
        // Maybe the generation step is being called shortly after the reset
        if (copyRuntime) {
            await fs_1.default.promises.cp(path_1.default.resolve(__dirname, '../../runtime'), config.runtimePath, {
                recursive: true
            });
        }
        core_1.ux.action.stop((0, chalk_1.default) `{cyan .${path_1.default.sep}${path_1.default.relative(config.cwd, config.runtimePath)}}`);
    }
    /**
     * If the user's source code imports any type only available inside the generated runtime, the first build will fail
     * because these imports resolves to an inexistent file, by running the parser twice we can avoid this problem.
     */
    async prepareFirstRun(config, compilerOptions) {
        core_1.ux.action.start('Searching runtime', '', {
            stdout: true,
            style: 'clock'
        });
        if (await fs_1.default.promises.access(path_1.default.join(config.runtimePath, 'plugin.js')).then(() => true, () => false)) {
            core_1.ux.action.stop((0, chalk_1.default) `{cyan Found!}`);
            return;
        }
        const formatter = new generator_1.KitaFormatter(config, true);
        const parser = parser_1.KitaParser.create(config, compilerOptions, compilerOptions.rootNames, formatter);
        // Ignores all errors, just generates something
        for await (const _ of parser.parse())
            ;
        await formatter.flush();
        core_1.ux.action.stop((0, chalk_1.default) `{yellow First run!}`);
    }
    async runParser(parser, formatter, reset, config) {
        core_1.ux.action.start('Parsing sources', '', {
            stdout: true,
            style: 'clock'
        });
        const diagnostics = [];
        // Should not emit any errors
        for await (const error of parser.parse()) {
            diagnostics.push(error.diagnostic);
        }
        core_1.ux.action.stop((0, diagnostics_1.formatStatus)(parser, diagnostics));
        if (diagnostics.length) {
            this.log((0, diagnostics_1.formatDiagnostic)(diagnostics));
        }
        if (formatter) {
            if (reset && config) {
                await this.resetRuntime(config, false);
            }
            core_1.ux.action.start((0, chalk_1.default) `Generating {cyan @kitajs/runtime}`, '', {
                stdout: true,
                style: 'clock'
            });
            const writeCount = await formatter.flush();
            core_1.ux.action.stop((0, chalk_1.default) `{${config?.declaration ? 'blue' : 'yellow'} ${writeCount}} files written.`);
        }
        else {
            this.log((0, chalk_1.default) `{yellow Skipping generation process.}`);
        }
        return diagnostics;
    }
    async catch(error) {
        // Pretty handle kita errors
        if (error instanceof common_1.KitaError) {
            this.logToStderr((0, diagnostics_1.formatDiagnostic)([error.diagnostic]));
            this.exit(1);
        }
        throw error;
    }
}
exports.BaseKitaCommand = BaseKitaCommand;
//# sourceMappingURL=base.js.map